//
//  Partition Logic
//  Copyright (C) 1998-2015 J. Andrew McLaughlin
//
//  This program is free software; you can redistribute it and/or modify it
//  under the terms of the GNU General Public License as published by the Free
//  Software Foundation; either version 2 of the License, or (at your option)
//  any later version.
//
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
//  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
//  for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
//
//  partlogic.c
//

#include <libintl.h>
#include <locale.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/api.h>
#include <sys/paths.h>

#define _(string) gettext(string)

#define WELCOME			_("Welcome to Partition Logic")
#define COPYRIGHT		_("Copyright (C) 1998-2015 J. Andrew McLaughlin")
#define GPLSHORT		_( \
	"  Contains software distributed under the terms of the GNU General  \n" \
	"  Public license - partitionlogic.org.uk")
#define GPL				_( \
	"  This program is free software; you can redistribute it and/or modify  \n" \
	"  it under the terms of the GNU General Public License as published by\n" \
	"  the Free Software Foundation; either version 2 of the License, or\n" \
	"  (at your option) any later version.\n\n" \
	"  This program is distributed in the hope that it will be useful, but\n" \
	"  WITHOUT ANY WARRANTY; without even the implied warranty of\n" \
	"  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See\n" \
	"  the file /system/COPYING.txt for more details.")
#define GRAPHICSMODE	_("Boot in graphics mode")
#define OK				_("OK")
#define CANCEL			_("Cancel")
#define LANGUAGE		_("Language")

// GUI stuff
static int graphics = 0;
static int bootGraphicsMode = 0;
static objectKey window = NULL;
static objectKey copyrightLabel = NULL;
static objectKey gplLabel = NULL;
static objectKey graphicsCheckbox = NULL;
static objectKey okButton = NULL;
static objectKey cancelButton = NULL;
static objectKey langButton = NULL;
static image flagImage;


static void setGraphics(int yes)
{
	// This allows the user to toggle text/graphics mode for the next reboot.

	file theFile;
	static char *noGraphName = "/nograph";

	if (yes)
	{
		// Try to delete any 'no graphics' file
		if (fileFind(noGraphName, &theFile) >= 0)
			fileDelete(noGraphName);
	}

	else
	{
		// Try to create a 'no graphics' file
		bzero(&theFile, sizeof(file));
		if (fileFind(noGraphName, &theFile) < 0)
		{
			fileOpen(noGraphName, (OPENMODE_WRITE | OPENMODE_CREATE), &theFile);
			fileClose(&theFile);
		}
	}
}


static int loadFlagImage(const char *lang, image *img)
{
	int status = 0;
	char path[MAX_PATH_LENGTH];
	file f;

	sprintf(path, PATH_SYSTEM_LOCALE "/flag-%s.bmp", lang);

	status = fileFind(path, &f);
	if (status < 0)
		return (status);

	status = imageLoad(path, 30, 20, img);

	return (status);
}


static void refreshWindow(void)
{
	char title[80];

	// Re-get the language setting
	setlocale(LC_ALL, getenv("LANG"));
	textdomain("partlogic");

	// Refresh the 'copyright' label
	windowComponentSetData(copyrightLabel, COPYRIGHT, strlen(COPYRIGHT));

	// Refresh the 'gpl' label
	windowComponentSetData(gplLabel, GPL, strlen(GPL));

	// Refresh the 'ok' button
	windowComponentSetData(okButton, OK, strlen(OK));

	// Refresh the 'cancel' button
	windowComponentSetData(cancelButton, CANCEL, strlen(CANCEL));

	if (langButton)
	{
		// Refresh the 'language' button
		if (flagImage.data)
			imageFree(&flagImage);
		if (loadFlagImage(getenv("LANG"), &flagImage) >= 0)
			windowComponentSetData(langButton, &flagImage, sizeof(image));
	}

	// Refresh the 'graphics mode' checkbox
	windowComponentSetData(graphicsCheckbox, GRAPHICSMODE,
		strlen(GRAPHICSMODE));

	// Refresh the window title
	sprintf(title, "%s %s", WELCOME, RELEASE);
	windowSetTitle(window, title);

	// Re-do the window layout
	windowLayout(window);
}


static void chooseLanguage(void)
{
	char pickedLanguage[6];

	if (windowNewLanguageDialog(window, pickedLanguage) >= 0)
	{
		setenv("LANG", pickedLanguage, 1);
		refreshWindow();
	}
}


static void eventHandler(objectKey key, windowEvent *event)
{
	// Check for the graphics checkbox
	if ((key == graphicsCheckbox) && (event->type & EVENT_SELECTION))
	{
		windowComponentGetSelected(graphicsCheckbox, &bootGraphicsMode);
		setGraphics(bootGraphicsMode);
	}

	// Check for the 'Ok' button
	if ((key == okButton) && (event->type == EVENT_MOUSE_LEFTUP))
		windowGuiStop();

	// Check for the 'Cancel' button
	else if ((key == cancelButton) && (event->type == EVENT_MOUSE_LEFTUP))
		shutdown(1, 0);

	// Check for the 'language' button
	else if (langButton && (key == langButton) &&
		(event->type == EVENT_MOUSE_LEFTUP))
	{
		chooseLanguage();
	}
}


static void constructWindow(const char *windowTitle)
{
	// If we are in graphics mode, make a window rather than operating on the
	// command line.

	int status = 0;
	image splashImage;
	objectKey font = NULL;
	objectKey buttonContainer = NULL;
	file langDir;
	int haveLocale = 0;
	componentParameters params;

	// Create a new window
	window = windowNew(multitaskerGetCurrentProcessId(), windowTitle);
	if (!window)
		return;

	bzero(&params, sizeof(componentParameters));
	params.gridWidth = 1;
	params.gridHeight = 1;
	params.padTop = 5;
	params.padLeft = 5;
	params.padRight = 5;
	params.orientationX = orient_left;
	params.orientationY = orient_middle;
	copyrightLabel = windowNewTextLabel(window, COPYRIGHT, &params);

	params.gridY += 1;
	params.padBottom = 5;
	params.orientationX = orient_center;
	params.flags |= WINDOW_COMPFLAG_HASBORDER;
	bzero(&splashImage, sizeof(image));
	imageLoad(PATH_SYSTEM "/partlogic.jpg", 0, 0, &splashImage);
	if (splashImage.data)
		// Create an image component from it, and add it to the window
		windowNewImage(window, &splashImage, draw_translucent, &params);

	params.gridY += 1;
	params.padBottom = 0;
	params.orientationX = orient_left;
	fontLoad("arial-bold-10.bmp", "arial-bold-10", &font, 0);
	params.flags &= ~WINDOW_COMPFLAG_HASBORDER;
	params.font = font;
	gplLabel = windowNewTextLabel(window, GPL, &params);

	params.gridY += 1;
	graphicsCheckbox = windowNewCheckbox(window, GRAPHICSMODE, &params);
	windowComponentSetSelected(graphicsCheckbox, bootGraphicsMode);
	windowRegisterEventHandler(graphicsCheckbox, &eventHandler);

	params.gridY += 1;
	params.orientationX = orient_center;
	params.flags |= (WINDOW_COMPFLAG_FIXEDWIDTH | WINDOW_COMPFLAG_FIXEDHEIGHT);
	buttonContainer = windowNewContainer(window, "buttonContainer", &params);
	if (buttonContainer)
	{
		params.padBottom = 5;
		params.orientationX = orient_right;
		params.flags |= WINDOW_COMPFLAG_FIXEDWIDTH;
		params.font = NULL;
		okButton = windowNewButton(buttonContainer, OK, NULL, &params);
		windowRegisterEventHandler(okButton, &eventHandler);
		windowComponentFocus(okButton);

		// Does the 'locale' directory exist?  Anything in it?
		if (fileFind(PATH_SYSTEM_LOCALE, &langDir) >= 0)
			haveLocale = 1;

		params.gridX += 1;
		if (haveLocale)
			params.orientationX = orient_center;
		else
			params.orientationX = orient_left;
		cancelButton = windowNewButton(buttonContainer, CANCEL, NULL, &params);
		windowRegisterEventHandler(cancelButton, &eventHandler);

		if (haveLocale)
		{
			params.gridX += 1;
			params.orientationX = orient_left;
			if (getenv("LANG"))
				status = loadFlagImage(getenv("LANG"), &flagImage);
			else
				status = loadFlagImage("en", &flagImage);
			if (status >= 0)
				langButton = windowNewButton(buttonContainer, NULL, &flagImage,
					&params);
			else
				langButton = windowNewButton(buttonContainer, LANGUAGE, NULL,
					&params);
			windowRegisterEventHandler(langButton, &eventHandler);
		}
	}

	// No minimize or close buttons on the window
	windowRemoveMinimizeButton(window);
	windowRemoveCloseButton(window);

	// Go
	windowSetVisible(window, 1);
}


int main(void)
{
	int diskManagerPid = 0;
	int logoWidth = 0;
	int logoPad = 0;
	int logoPos = 0;
	char title[80];
	char tmpChar[80];

	graphics = graphicsAreEnabled();
	bootGraphicsMode = graphics;

	setlocale(LC_ALL, getenv("LANG"));
	textdomain("partlogic");

	userLogin("admin", "");

	sprintf(title, "%s %s", WELCOME, RELEASE);
	logoWidth = (strlen(title) + 10);

	if (graphics)
	{
		constructWindow(title);
		windowGuiRun();
		windowLogin("admin");
		windowDestroy(window);
	}

	else
	{
	restart:
		textScreenClear();

		// Use some extended ascii to make it prettier in text mode.  This
		// is convoluted -- not much use trying to understand it.
		logoPad = ((((logoWidth - strlen(title)) / 2)) - 1);
		logoPos = ((80 - logoWidth) / 2);
		memset(tmpChar, ' ', logoPos);
		tmpChar[logoPos++] = 201;
		memset((tmpChar + logoPos), 205, (logoWidth - 2));
		logoPos += (logoWidth - 2);
		tmpChar[logoPos++] = 187;
		tmpChar[logoPos++] = '\0';
		printf("\n%s\n", tmpChar);
		logoPos = ((80 - logoWidth) / 2);
		memset(tmpChar, ' ', logoPos);
		tmpChar[logoPos++] = 186;
		memset((tmpChar + logoPos), ' ', logoPad);
		logoPos += logoPad;
		strncpy((tmpChar + logoPos), title, strlen(title));
		logoPos += strlen(title);
		memset((tmpChar + logoPos), ' ', logoPad);
		logoPos += logoPad;
		tmpChar[logoPos++] = 186;
		tmpChar[logoPos++] = '\0';
		printf("%s\n", tmpChar);
		logoPos = ((80 - logoWidth) / 2);
		memset(tmpChar, ' ', logoPos);
		tmpChar[logoPos++] = 200;
		memset((tmpChar + logoPos), 205, (logoWidth - 2));
		logoPos += (logoWidth - 2);
		tmpChar[logoPos++] = 188;
		tmpChar[logoPos++] = '\0';
		printf("%s\n\n", tmpChar);
		printf("%s\n\n", GPL);
		printf(_("Boot mode: %s [ Press 'G' to change at next boot ]\n\n"
			"Continue? (y/n): "),
			(bootGraphicsMode? _("GRAPHICS") : _("TEXT    ")));

		while (1)
		{
			textInputSetEcho(0);
			char c = getchar();
			textInputSetEcho(1);

			switch (c)
			{
				case 'y':
				case 'Y':
					printf("%s", _("Yes\n"));
					break;
				case 'n':
				case 'N':
					printf("%s", _("No\n"));
					shutdown(1, 0);
				case 'g':
				case 'G':
					if (bootGraphicsMode)
						bootGraphicsMode = 0;
					else
						bootGraphicsMode = 1;
					setGraphics(bootGraphicsMode);
					goto restart;
			}

			break;
		}
	}

	while(1)
	{
		// Loop here, so that if the user closes the disk manager and then
		// cancels the shutdown, we will start a new disk manager.

		diskManagerPid = loaderLoadProgram(PATH_PROGRAMS "/fdisk", 0);

		// Set the PID to the window manager thread
		userSetPid("admin", diskManagerPid);

		loaderExecProgram(diskManagerPid, 1);

		// If we return to here, the user has closed the disk manager.
		if (graphics)
			loaderLoadAndExec(PATH_PROGRAMS "/shutdown", 0, 1);
		else
			shutdown(1, 0);
	}
}

